//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_FILTER_TIMESTAMP_H
#define LIBUNCIA_OUTPUT_FILTER_TIMESTAMP_H

#include <libuncia/ostream.h>
#include <libuncia/output/filter.h>

namespace libuncia
{
    /**
      * The output_filter_timestamp class is used to represent an output
      * filter which prepends a time stamp to each output line.  Works
      * best when input is line buffered.  Often useful filter for error
      * or logging output streams.
      */
    class output_filter_timestamp:
        public output_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_filter_timestamp();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to send the output of this filter.
          * @param strftime_format
          *     the strftime format to be used for the timestamp.
          *     See strftime(3) for more information.
          */
        output_filter_timestamp(const pointer &deeper,
            const std::string &strftime_format);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to send the output of this filter.
          */
        static pointer create(const pointer &deeper);

        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to send the output of this filter.
          * @param strftime_format
          *     the strftime format to be used for the timestamp.
          *     See strftime(3) for more information.
          */
        static pointer create(const pointer &deeper,
            const std::string &strftime_format);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

    private:
        /**
          * The strftime_format instance variable is used to remember the
          * strftime format to be used for the timestamp.
          * See strftime(3) for more information.
          */
        std::string strftime_format;

         /**
           * The start_of_line instance variable is used to remember
           * whether or not the current output position is at the start
           * of a line.
           */
        bool start_of_line;

        /**
          * The default constructor.  Do not use.
          */
        output_filter_timestamp();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_filter_timestamp(const output_filter_timestamp &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_filter_timestamp &operator=(const output_filter_timestamp &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_FILTER_TIMESTAMP_H
